@@ -8,10 +8,13 @@
 from typing import Any, Generic, Optional, Set, TypeVar
 
 import networkx as nx
-from bytecode import BasicBlock
+from bytecode import UNSET, BasicBlock, Compare
 from networkx import lowest_common_ancestor
 from networkx.drawing.nx_pydot import to_pydot
 
+# Key for storing branch value in networkx edge.
+EDGE_DATA_BRANCH_VALUE = "branch_value"
+
 
 class ProgramGraphNode:
     """A base class for a node of the program graph."""
@@ -84,7 +87,28 @@ class ProgramGraphNode:
         return 31 + 17 * self._index
 
     def __str__(self) -> str:
-        return f"ProgramGraphNode({self._index})"
+        result = f"ProgramGraphNode({self._index})"
+        if self._predicate_id is not None:
+            result += f"\npredicate_id {self._predicate_id}"
+        if self._basic_block is not None:
+            instructions = []
+            for instr in self._basic_block:
+                arg = instr.arg
+                if isinstance(arg, BasicBlock):
+                    # We cannot determine which ProgramGraphNode this is.
+                    arg = "ProgramGraphNode"
+                elif isinstance(arg, Compare):
+                    arg = arg.name
+                elif arg is UNSET:
+                    arg = ""
+                else:
+                    arg = repr(arg)
+                formatted = instr.name
+                if arg != "":
+                    formatted += f" {arg}"
+                instructions.append(formatted)
+            result += "\n" + "\n".join(instructions)
+        return result
 
     def __repr__(self) -> str:
         return f"ProgramGraphNode(index={self._index}, basic_block={self._basic_block})"
