@@ -23,6 +23,7 @@ from pynguin.utils.generic.genericaccessibleobject import (
 )
 
 if TYPE_CHECKING:
+    import pynguin.testcase.defaulttestcase as dtc
     import pynguin.testcase.testcase as tc
     import pynguin.testcase.variablereference as vr
     from pynguin.assertion.assertion import Assertion
@@ -36,7 +37,7 @@ def create_assign_stmt(
     testcase: tc.TestCase,
     ref_dict: Dict[str, vr.VariableReference],
     test_cluster: TestCluster,
-) -> Optional[Tuple[str, stmt.Statement]]:
+) -> Optional[Tuple[str, stmt.VariableCreatingStatement]]:
     """Creates the corresponding statement from an ast.Assign node.
 
     Args:
@@ -49,7 +50,7 @@ def create_assign_stmt(
     Returns:
         The corresponding statement or None if no statement type matches.
     """
-    new_stmt: Optional[stmt.Statement]
+    new_stmt: Optional[stmt.VariableCreatingStatement]
     value = assign.value
     objs_under_test = test_cluster.accessible_objects_under_test
     callable_objects_under_test: Set[GenericCallableAccessibleObject] = {
@@ -194,7 +195,7 @@ def create_variable_references_from_call_args(
 # pylint: disable=too-many-return-statements
 def create_stmt_from_constant(
     constant: ast.Constant, testcase: tc.TestCase
-) -> Optional[stmt.PrimitiveStatement]:
+) -> Optional[stmt.VariableCreatingStatement]:
     """Creates a statement from an ast.constant node.
 
     Args:
@@ -224,7 +225,7 @@ def create_stmt_from_constant(
 
 def create_stmt_from_unaryop(
     unaryop: ast.UnaryOp, testcase: tc.TestCase
-) -> Optional[stmt.PrimitiveStatement]:
+) -> Optional[stmt.VariableCreatingStatement]:
     """Creates a statement from an ast.unaryop node.
 
     Args:
@@ -252,7 +253,7 @@ def create_stmt_from_call(
     testcase: tc.TestCase,
     objs_under_test: Set[GenericCallableAccessibleObject],
     ref_dict: Dict[str, vr.VariableReference],
-) -> Optional[Union[stmt.CollectionStatement, stmt.ParametrizedStatement]]:
+) -> Optional[stmt.VariableCreatingStatement]:
     """Creates the corresponding statement from an ast.call node. Depending on the call,
     this can be a GenericConstructor, GenericMethod or GenericFunction statement.
 
@@ -324,7 +325,7 @@ def find_gen_callable(
             call_id = call.func.value.id  # type: ignore
             if call_name == obj.method_name and call_id in ref_dict:
                 obj_from_ast = str(call.func.value.id)  # type: ignore
-                var_type = ref_dict[obj_from_ast].variable_type
+                var_type = ref_dict[obj_from_ast].type
                 if var_type == obj.owner:
                     return obj
         elif isinstance(obj, GenericFunction):
@@ -386,11 +387,7 @@ def create_stmt_from_collection(
     testcase: tc.TestCase,
     objs_under_test: Set[GenericCallableAccessibleObject],
     ref_dict: Dict[str, vr.VariableReference],
-) -> Optional[
-    Union[
-        stmt.ListStatement, stmt.SetStatement, stmt.DictStatement, stmt.TupleStatement
-    ]
-]:
+) -> Optional[stmt.VariableCreatingStatement]:
     """Creates the corresponding statement from an ast.List node. Lists contain other
     statements.
 
@@ -453,35 +450,29 @@ def create_elements(
     """
     coll_elems: List[vr.VariableReference] = []
     for elem in elements:
-        statement: Optional[
-            Union[
-                stmt.PrimitiveStatement,
-                stmt.CollectionStatement,
-                stmt.ParametrizedStatement,
-            ]
-        ]
+        statement: Optional[stmt.VariableCreatingStatement]
         if isinstance(elem, ast.Constant):
             statement = create_stmt_from_constant(elem, testcase)
             if not statement:
                 return None
-            coll_elems.append(testcase.add_statement(statement))
+            coll_elems.append(testcase.add_variable_creating_statement(statement))
         elif isinstance(elem, ast.UnaryOp):
             statement = create_stmt_from_unaryop(elem, testcase)
             if not statement:
                 return None
-            coll_elems.append(testcase.add_statement(statement))
+            coll_elems.append(testcase.add_variable_creating_statement(statement))
         elif isinstance(elem, ast.Call):
             statement = create_stmt_from_call(elem, testcase, objs_under_test, ref_dict)
             if not statement:
                 return None
-            coll_elems.append(testcase.add_statement(statement))
+            coll_elems.append(testcase.add_variable_creating_statement(statement))
         elif isinstance(elem, (ast.List, ast.Tuple, ast.Set, ast.Dict)):
             statement = create_stmt_from_collection(
                 elem, testcase, objs_under_test, ref_dict
             )
             if not statement:
                 return None
-            coll_elems.append(testcase.add_statement(statement))
+            coll_elems.append(testcase.add_variable_creating_statement(statement))
         elif isinstance(elem, ast.Name):
             try:
                 coll_elems.append(ref_dict[elem.id])
@@ -504,9 +495,9 @@ def get_collection_type(coll_elems: List[vr.VariableReference]) -> Any:
     """
     if len(coll_elems) == 0:
         return None
-    coll_type = coll_elems[0].variable_type
+    coll_type = coll_elems[0].type
     for elem in coll_elems:
-        if not elem.variable_type == coll_type:
+        if not elem.type == coll_type:
             coll_type = None
             break
     return coll_type
@@ -550,7 +541,7 @@ def try_generating_specific_function(
     testcase: tc.TestCase,
     objs_under_test: Set[GenericCallableAccessibleObject],
     ref_dict: Dict[str, vr.VariableReference],
-) -> Optional[stmt.CollectionStatement]:
+) -> Optional[stmt.VariableCreatingStatement]:
     """Calls to creating a collection (list, set, tuple, dict) via their keywords and
     not via literal syntax are considered as ast.Call statements. But for these calls,
     no accessible object under test is in the test_cluster. To parse them anyway, these
@@ -618,3 +609,59 @@ def try_generating_specific_function(
             dict_node, testcase, objs_under_test, ref_dict
         )
     return None
+
+
+# pylint: disable=invalid-name, missing-function-docstring
+class AstToTestCaseTransformer(ast.NodeVisitor):
+    """A AST NodeVisitor that tries to convert an AST into our internal
+    test case representation."""
+
+    def __init__(self, test_cluster: TestCluster, create_assertions: bool):
+        self._current_testcase: dtc.DefaultTestCase = dtc.DefaultTestCase()
+        self._current_parsable: bool = True
+        self._var_refs: Dict[str, vr.VariableReference] = {}
+        self._testcases: List[dtc.DefaultTestCase] = []
+        self._number_found_testcases: int = 0
+        self._test_cluster = test_cluster
+        self._create_assertions = create_assertions
+
+    def visit_FunctionDef(self, node: ast.FunctionDef) -> Any:
+        self._number_found_testcases += 1
+        self._current_testcase = dtc.DefaultTestCase()
+        self._current_parsable = True
+        self._var_refs.clear()
+        self.generic_visit(node)
+        if self._current_parsable:
+            self._testcases.append(self._current_testcase)
+
+    def visit_Assign(self, node: ast.Assign) -> Any:
+        if self._current_parsable:
+            if (
+                result := create_assign_stmt(
+                    node, self._current_testcase, self._var_refs, self._test_cluster
+                )
+            ) is None:
+                self._current_parsable = False
+            else:
+                ref_id, stm = result
+                var_ref = self._current_testcase.add_variable_creating_statement(stm)
+                self._var_refs[ref_id] = var_ref
+
+    def visit_Assert(self, node: ast.Assert) -> Any:
+        if self._current_parsable and self._create_assertions:
+            if (result := create_assert_stmt(self._var_refs, node)) is not None:
+                assertion, var_ref = result
+                self._current_testcase.get_statement(
+                    var_ref.get_statement_position()
+                ).add_assertion(assertion)
+
+    @property
+    def testcases(self) -> List[dtc.DefaultTestCase]:
+        """Provides the testcases that could be generated from the given AST.
+        It is possible that not every aspect of the AST could be transformed
+        to our internal representation.
+
+        Returns:
+            The generated testcases.
+        """
+        return self._testcases
