@@ -9,15 +9,15 @@ from __future__ import annotations
 
 from typing import TYPE_CHECKING, Any, Set
 
-import pynguin.assertion.outputtraceentry as ote
 import pynguin.assertion.primitiveassertion as pas
+import pynguin.assertion.statetraceentry as ote
 
 if TYPE_CHECKING:
     import pynguin.assertion.assertion as ass
     import pynguin.testcase.variablereference as vr
 
 
-class PrimitiveTraceEntry(ote.OutputTraceEntry):
+class PrimitiveTraceEntry(ote.StateTraceEntry):
     """An entry for a primitive assertion."""
 
     def __init__(self, variable: vr.VariableReference, value: Any) -> None:
