@@ -5,21 +5,21 @@
 #  SPDX-License-Identifier: LGPL-3.0-or-later
 #
 """Provides a MIO."""
+from __future__ import annotations
+
 import logging
 from dataclasses import dataclass
 from math import ceil
-from typing import List, Optional, cast
+from typing import TYPE_CHECKING, Optional
 
 import pynguin.configuration as config
-import pynguin.ga.chromosome as chrom
-import pynguin.ga.fitnessfunctions.abstracttestcasefitnessfunction as atcff
-import pynguin.ga.testcasechromosome as tcc
-import pynguin.generation.algorithms.mioarchive as mioa
-import pynguin.utils.statistics.statistics as stat
+import pynguin.generation.algorithms.archive as arch
 from pynguin.generation.algorithms.testgenerationstrategy import TestGenerationStrategy
-from pynguin.generation.algorithms.wraptestsuitemixin import WrapTestSuiteMixin
 from pynguin.utils import randomness
-from pynguin.utils.statistics.runtimevariable import RuntimeVariable
+
+if TYPE_CHECKING:
+    import pynguin.ga.testcasechromosome as tcc
+    import pynguin.ga.testsuitechromosome as tsc
 
 
 # pylint: disable=invalid-name
@@ -46,7 +46,7 @@ class Parameters:
 
 
 # pylint: disable=too-few-public-methods
-class MIOTestStrategy(TestGenerationStrategy, WrapTestSuiteMixin):
+class MIOTestStrategy(TestGenerationStrategy[arch.MIOArchive]):
     """Implements MIO."""
 
     _logger = logging.getLogger(__name__)
@@ -54,36 +54,25 @@ class MIOTestStrategy(TestGenerationStrategy, WrapTestSuiteMixin):
     def __init__(self) -> None:
         super().__init__()
         self._solution: Optional[tcc.TestCaseChromosome] = None
-        self._archive: mioa.MIOArchive
         self._parameters = Parameters()
         self._current_mutations = 0
         self._focused = False
 
     def generate_tests(
         self,
-    ) -> chrom.Chromosome:
-        self._archive = mioa.MIOArchive(
-            cast(List[atcff.AbstractTestCaseFitnessFunction], self.fitness_functions),
-            self._parameters.n,
-        )
-        generation = 0
+    ) -> tsc.TestSuiteChromosome:
+        self.before_search_start()
         while (
-            not self._stopping_condition.is_fulfilled()
-            and len(self.fitness_functions) - self._archive.num_covered_targets != 0
+            self.resources_left()
+            and len(self._test_case_fitness_functions)
+            - self._archive.num_covered_targets
+            != 0
         ):
             self.evolve()
-            test_suite = self.create_test_suite(self._archive.get_solutions())
-            stat.current_individual(test_suite)
-            self._logger.info(
-                "Generation: %5i. Best fitness: %5f, Best coverage %5f",
-                generation,
-                test_suite.get_fitness(),
-                test_suite.get_coverage(),
-            )
             self._update_parameters()
-            generation += 1
-        stat.track_output_variable(RuntimeVariable.AlgorithmIterations, generation)
-        return self.create_test_suite(self._archive.get_solutions())
+            self.after_search_iteration(self.create_test_suite(self._archive.solutions))
+        self.after_search_finish()
+        return self.create_test_suite(self._archive.solutions)
 
     # pylint:disable=line-too-long
     def _update_parameters(self):
@@ -154,21 +143,15 @@ class MIOTestStrategy(TestGenerationStrategy, WrapTestSuiteMixin):
             self._current_mutations += 1
         elif randomness.next_float() < self._parameters.Pr:
             offspring = self.chromosome_factory.get_chromosome()
-            self._add_fitness_functions(offspring)
             self._current_mutations = 1
         else:
             maybe_offspring = self._archive.get_solution()
             if maybe_offspring is None:
                 # Nothing in archive, so sample new one.
                 offspring = self.chromosome_factory.get_chromosome()
-                self._add_fitness_functions(offspring)
             else:
                 offspring = maybe_offspring
             offspring.mutate()
             self._current_mutations = 1
-        if self._archive.update_archive(offspring):
+        if self._archive.update([offspring]):
             self._solution = offspring
-
-    def _add_fitness_functions(self, chromosome: tcc.TestCaseChromosome) -> None:
-        for fitness_function in self._fitness_functions:
-            chromosome.add_fitness_function(fitness_function)
