@@ -16,36 +16,44 @@ framework.
 Pynguin is supposed to be used as a standalone command-line application but it
 can also be used as a library by instantiating this class directly.
 """
+from __future__ import annotations
+
+import datetime
 import enum
 import importlib
 import logging
 import os
 import sys
 import threading
-import time
-from typing import List, Optional, Tuple
+from pathlib import Path
+from typing import TYPE_CHECKING, List, Optional, Tuple
 
 import pynguin.analyses.seeding.initialpopulationseeding as initpopseeding
 import pynguin.assertion.assertiongenerator as ag
+import pynguin.assertion.mutation_analysis.mutationanalysisgenerator as mag
 import pynguin.configuration as config
 import pynguin.ga.chromosome as chrom
 import pynguin.ga.chromosomeconverter as cc
+import pynguin.ga.chromosomevisitor as cv
+import pynguin.ga.computations as ff
 import pynguin.ga.postprocess as pp
 import pynguin.generation.generationalgorithmfactory as gaf
 import pynguin.testcase.testcase as tc
 import pynguin.utils.statistics.statistics as stat
 from pynguin.analyses.seeding.constantseeding import static_constant_seeding
-from pynguin.ga.fitnessfunctions.fitness_utilities import compute_branch_coverage
-from pynguin.generation.algorithms.testgenerationstrategy import TestGenerationStrategy
 from pynguin.generation.export.exportprovider import ExportProvider
 from pynguin.instrumentation.machinery import install_import_hook
-from pynguin.setup.testcluster import TestCluster
 from pynguin.setup.testclustergenerator import TestClusterGenerator
-from pynguin.testcase.execution.executiontracer import ExecutionTracer
-from pynguin.testcase.execution.testcaseexecutor import TestCaseExecutor
+from pynguin.testcase.execution import ExecutionTracer, TestCaseExecutor
 from pynguin.utils import randomness
+from pynguin.utils.report import get_coverage_report, render_coverage_report
 from pynguin.utils.statistics.runtimevariable import RuntimeVariable
-from pynguin.utils.statistics.timer import Timer
+
+if TYPE_CHECKING:
+    from pynguin.generation.algorithms.testgenerationstrategy import (
+        TestGenerationStrategy,
+    )
+    from pynguin.setup.testcluster import TestCluster
 
 
 @enum.unique
@@ -93,14 +101,13 @@ def run_pynguin() -> ReturnCode:
 
 
 def _setup_test_cluster() -> Optional[TestCluster]:
-    with Timer(name="Test-cluster generation time", logger=None):
-        test_cluster = TestClusterGenerator(
-            config.configuration.module_name
-        ).generate_cluster()
-        if test_cluster.num_accessible_objects_under_test() == 0:
-            _LOGGER.error("SUT contains nothing we can test.")
-            return None
-        return test_cluster
+    test_cluster = TestClusterGenerator(
+        config.configuration.module_name
+    ).generate_cluster()
+    if test_cluster.num_accessible_objects_under_test() == 0:
+        _LOGGER.error("SUT contains nothing we can test.")
+        return None
+    return test_cluster
 
 
 def _setup_path() -> bool:
@@ -130,7 +137,7 @@ def _setup_import_hook() -> ExecutionTracer:
 def _load_sut(tracer: ExecutionTracer) -> bool:
     try:
         # We need to set the current thread ident so the import trace is recorded.
-        tracer.current_thread_ident = threading.currentThread().ident
+        tracer.current_thread_identifier = threading.current_thread().ident
         importlib.import_module(config.configuration.module_name)
     except ImportError as ex:
         # A module could not be imported because some dependencies
@@ -140,6 +147,26 @@ def _load_sut(tracer: ExecutionTracer) -> bool:
     return True
 
 
+def _setup_report_dir() -> bool:
+    # Report dir only needs to be created when statistics or coverage report is enabled.
+    if (
+        config.configuration.statistics_output.statistics_backend
+        != config.StatisticsBackend.NONE
+        or config.configuration.statistics_output.create_coverage_report
+    ):
+        report_dir = Path(config.configuration.statistics_output.report_dir).absolute()
+        try:
+            report_dir.mkdir(parents=True, exist_ok=True)
+        except (OSError, FileNotFoundError):
+            _LOGGER.error(
+                "Cannot create report dir %s",
+                config.configuration.statistics_output.report_dir,
+                exc_info=True,
+            )
+            return False
+    return True
+
+
 def _setup_random_number_generator() -> None:
     """Setup RNG."""
     if config.configuration.seeding.seed is None:
@@ -180,8 +207,15 @@ def _setup_and_check() -> Optional[Tuple[TestCaseExecutor, TestCluster]]:
     tracer = _setup_import_hook()
     if not _load_sut(tracer):
         return None
+    if not _setup_report_dir():
+        return None
+
+    # Analyzing the SUT should not cause any coverage.
+    tracer.disable()
     if (test_cluster := _setup_test_cluster()) is None:
         return None
+    tracer.enable()
+
     executor = TestCaseExecutor(tracer)
     _track_sut_data(tracer, test_cluster)
     _setup_random_number_generator()
@@ -216,7 +250,7 @@ def _track_sut_data(tracer: ExecutionTracer, test_cluster: TestCluster) -> None:
     # TODO(fk) make this work for other criteria beyond branch coverage.
     stat.track_output_variable(
         RuntimeVariable.ImportBranchCoverage,
-        compute_branch_coverage(tracer.import_trace, tracer.get_known_data()),
+        ff.compute_branch_coverage(tracer.import_trace, tracer.get_known_data()),
     )
 
 
@@ -225,56 +259,71 @@ def _run() -> ReturnCode:
         return ReturnCode.SETUP_FAILED
     executor, test_cluster = setup_result
 
-    with Timer(name="Test generation time", logger=None):
-        algorithm: TestGenerationStrategy = _instantiate_test_generation_strategy(
-            executor, test_cluster
-        )
-        _LOGGER.info(
-            "Start generating sequences using %s", config.configuration.algorithm
-        )
-        stat.set_sequence_start_time(time.time_ns())
-        generation_result = algorithm.generate_tests()
-        _LOGGER.info(
-            "Stop generating sequences using %s", config.configuration.algorithm
-        )
-        algorithm.send_statistics()
+    algorithm: TestGenerationStrategy = _instantiate_test_generation_strategy(
+        executor, test_cluster
+    )
+    _LOGGER.info("Start generating test cases")
+    generation_result = algorithm.generate_tests()
+    if algorithm.stopping_condition.is_fulfilled():
+        _LOGGER.info("Used up all resources (%s).", algorithm.stopping_condition)
+    else:
+        _LOGGER.info("Algorithm stopped before using all resources.")
+    _LOGGER.info("Stop generating test cases")
 
-        with Timer(name="Re-execution time", logger=None):
-            stat.track_output_variable(
-                RuntimeVariable.Coverage, generation_result.get_coverage()
-            )
+    # Executions that happen after this point should not influence the
+    # search statistics
+    executor.clear_observers()
 
-        if config.configuration.test_case_output.post_process:
-            postprocessor = pp.ExceptionTruncation()
-            generation_result.accept(postprocessor)
-            # TODO(fk) add more postprocessing stuff.
+    # Track coverage of the generated test suites.
+    # This possibly re-executes the test suites.
+    stat.track_output_variable(
+        RuntimeVariable.Coverage, generation_result.get_coverage()
+    )
 
-        if config.configuration.test_case_output.generate_assertions:
+    if config.configuration.test_case_output.post_process:
+        truncation = pp.ExceptionTruncation()
+        generation_result.accept(truncation)
+
+        unused_primitives_removal = pp.TestCasePostProcessor(
+            [pp.UnusedStatementsTestCaseVisitor()]
+        )
+        generation_result.accept(unused_primitives_removal)
+        # TODO(fk) add more postprocessing stuff.
+
+    ass_gen = config.configuration.test_case_output.assertion_generation
+    if ass_gen != config.AssertionGenerator.NONE:
+        if ass_gen == config.AssertionGenerator.MUTATION_ANALYSIS:
+            generator: cv.ChromosomeVisitor = mag.MutationAnalysisGenerator(executor)
+        else:
             generator = ag.AssertionGenerator(executor)
-            generation_result.accept(generator)
+        generation_result.accept(generator)
 
-        with Timer(name="Export time", logger=None):
-            converter = cc.ChromosomeConverter()
-            generation_result.accept(converter)
-            failing = converter.failing_test_suite
-            passing = converter.passing_test_suite
-            written_to = _export_test_cases(
-                [t.test_case for t in passing.test_case_chromosomes]
-            )
-            _LOGGER.info(
-                "Export %i successful test cases to %s",
-                passing.size(),
-                written_to,
-            )
-            written_to = _export_test_cases(
-                [t.test_case for t in failing.test_case_chromosomes],
-                "_failing",
-                wrap_code=True,
-            )
-            _LOGGER.info(
-                "Export %i failing test cases to %s", failing.size(), written_to
-            )
+    # Export the generated test suites
+    converter = cc.ChromosomeConverter()
+    generation_result.accept(converter)
+    failing = converter.failing_test_suite
+    passing = converter.passing_test_suite
+    written_to = _export_test_cases(
+        [t.test_case for t in passing.test_case_chromosomes]
+    )
+    _LOGGER.info(
+        "Export %i successful test cases to %s",
+        passing.size(),
+        written_to,
+    )
+    written_to = _export_test_cases(
+        [t.test_case for t in failing.test_case_chromosomes],
+        "_failing",
+        wrap_code=True,
+    )
+    _LOGGER.info("Export %i failing test cases to %s", failing.size(), written_to)
 
+    if config.configuration.statistics_output.create_coverage_report:
+        render_coverage_report(
+            get_coverage_report(generation_result, executor),
+            Path(config.configuration.statistics_output.report_dir) / "cov_report.html",
+            datetime.datetime.now(),
+        )
     _track_statistics(passing, failing, generation_result)
     _collect_statistics()
     if not stat.write_statistics():
